import abc
from collections.abc import Callable

from django.http import HttpRequest, HttpResponse

from aioinject import SyncContainer
from aioinject._types import P, T
from aioinject.decorators import base_inject


def inject(function: Callable[P, T]) -> Callable[P, T]:
    return base_inject(
        function,
        context_parameters=(),
        context_getter=lambda args, kwargs: args[0].__aioinject_context__,  # noqa: ARG005
    )


class SyncAioinjectMiddleware(abc.ABC):
    def __init__(
        self, get_response: Callable[[HttpRequest], HttpResponse]
    ) -> None:
        self.get_response = get_response

    @property
    @abc.abstractmethod
    def container(self) -> SyncContainer:
        raise NotImplementedError

    def __call__(self, request: HttpRequest) -> HttpResponse:
        with self.container.context() as ctx:
            request.__aioinject_context__ = ctx  # type: ignore[attr-defined]
            return self.get_response(request)



import inspect
from collections.abc import Awaitable, Callable
from typing import Any

from aiogram import BaseMiddleware, Router
from aiogram.types import TelegramObject

import aioinject
from aioinject._types import P, T
from aioinject.decorators import add_parameters_to_signature, base_inject


__all__ = ["AioInjectMiddleware", "inject"]


_ARG_NAME = "aioinject_context"


def inject(function: Callable[P, T]) -> Callable[P, T]:  # pragma: no cover
    signature = inspect.signature(function)
    existing_parameter = signature.parameters.get(_ARG_NAME)
    if not existing_parameter:
        add_parameters_to_signature(function, {_ARG_NAME: aioinject.Context})

    return base_inject(
        function,
        context_parameters=(),
        context_getter=lambda args, kwargs: kwargs.pop(_ARG_NAME)  # noqa: ARG005
        if not existing_parameter
        else kwargs[_ARG_NAME],
    )


class AioInjectMiddleware(BaseMiddleware):
    def __init__(self, container: aioinject.Container) -> None:
        self.container = container

    async def __call__(
        self,
        handler: Callable[[TelegramObject, dict[str, Any]], Awaitable[Any]],
        event: TelegramObject,
        data: dict[str, Any],
    ) -> Any:
        async with self.container.context() as ctx:
            data[_ARG_NAME] = ctx
            return await handler(event, data)

    def add_to_router(self, router: Router) -> None:
        for observer in router.observers.values():
            observer.outer_middleware.register(middleware=self)



from aioinject import Container, Scoped


class _Interface:
    pass


class _A(_Interface):
    pass


class _Dependant:
    def __init__(self, interface: _Interface) -> None:
        self.interface = interface


async def test_ok() -> None:
    container = Container()
    container.register(Scoped(_A, interface=_Interface))
    container.register(Scoped(_Dependant))

    async with container.context() as context:
        result = await context.resolve(_Dependant)
        assert isinstance(result, _Dependant)
        assert isinstance(result.interface, _A)



from django.urls import path

from tests.integrations.django.app.test.views import (
    function_view,
    function_view_with_parameter,
)


urlpatterns = [
    path("dj/function", function_view),
    path("dj/function/<str:parameter>", function_view_with_parameter),
]
